import java.awt.Container;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextArea;

public class FahrzeugFrame extends JFrame implements ActionListener {

    private JButton erzeugenButton;
    private JButton loeschenButton;
    private JButton beschleunigenButton;

    private JTextArea ausgabeBereich;

    private Fahrzeug aktuellesFahrzeug;

    /**
     * In diesem Programmstück wird das Fenster erzeugt. Auf dem Fenster sind
     * später drei Buttons für Erzeugen, Löschen und Beschleunigen zu sehen.
     * Außerdem gibt es einen Ausgabebereich, in dem ausgegeben wird, welche
     * Aktion das Programm gerade ausgeführt hat.
     */
    public FahrzeugFrame() {
        /* In der Kopfleiste des Fenster steht "FahrzeugFrame" */
        this.setTitle("FahrzeugFrame");

        /* Das Fenster ist 700 Pixel breit und 200 Pixel hoch. */
        this.setSize(700, 200);

        /* Hier werden alle Buttons erzeugt. */
        this.erzeugenButton = new JButton("Erzeugen");
        this.loeschenButton = new JButton("Löschen");
        this.beschleunigenButton = new JButton("Beschleunigen");

        /* Hier wird der Ausgabe-Bereich erzeugt. */
        this.ausgabeBereich = new JTextArea(10, 100);

        /* Hier werden alle Buttons zusammengruppiert. */
        JPanel buttonPanel = new JPanel();
        buttonPanel.add(this.erzeugenButton);
        buttonPanel.add(this.loeschenButton);
        buttonPanel.add(this.beschleunigenButton);

        /*
         * Der ContentPane ist der Ausschnitt des Fensters, auf dem Widgets d.h.
         * Interaktionselemente (wie eine TextArea oder ein Button) platziert
         * werden können.
         */
        Container contentPane = this.getContentPane();
        contentPane.setLayout(new GridLayout(2, 1));
        /* Hier wird die Gruppe von Buttons platziert. */
        contentPane.add(buttonPanel);
        /* Hier wird der Ausgabebereich platziert. */
        contentPane.add(this.ausgabeBereich);

        /*
         * Hier wird der FahrzeugFrame als Listener für Knopfdruck-Ereignisse
         * bei jedem der drei Buttons registriert.
         */
        this.erzeugenButton.addActionListener(this);
        this.loeschenButton.addActionListener(this);
        this.beschleunigenButton.addActionListener(this);

        /*
         * Wird das Fenster geschlossen (d.h. auf X gedrückt), wird mit Hilfe
         * dieser Programmzeile auch unser Programm ordnungsgemäß beendet.
         */
        this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    }

    /**
     * Dieses Programmstück wird immer dann ausgeführt, wenn ein Benutzer auf
     * einen Button drückt.
     */
    @Override
    public void actionPerformed(ActionEvent e) {
        Object source = e.getSource();
        if (source == this.erzeugenButton) {
            this.erzeugen();
        }
        else if (source == this.loeschenButton) {
            this.loeschen();
        }
        else if (source == this.beschleunigenButton) {
            this.beschleunigen();
        }
    }

    /**
     * Diese Methode erzeugt ein neues Fahrzeug, falls es momentan kein Fahrzeug
     * gibt.
     */
    private void erzeugen() {
        if (this.aktuellesFahrzeug != null) {
            this.ausgabeBereich
                    .setText("Es existiert bereits ein Fahrzeug. "
                            + "Sie müssen dieses Fahrzeug zuerst löschen, "
                            + "bevor ein neues Fahrzeug erzeugt werden kann!");
        }
        else {
            String einlesenPositionX = JOptionPane
                    .showInputDialog("Aktuelle x-Position: ");
            int xpos = Integer.parseInt(einlesenPositionX);

            String einlesenPositionY = JOptionPane
                    .showInputDialog("Aktuelle y-Position: ");
            int ypos = Integer.parseInt(einlesenPositionY);

            String einlesenAnzahlRaeder = JOptionPane
                    .showInputDialog("Anzahl Räder: ");
            int anzahlRaeder = Integer.parseInt(einlesenAnzahlRaeder);

            String einlesenGewicht = JOptionPane
                    .showInputDialog("Gewicht: ");
            double gewicht = Double.parseDouble(einlesenGewicht);

            String einlesenGeschwindigkeit = JOptionPane
                    .showInputDialog("Geschwindigkeit: ");
            double geschwindigkeit = Double
                    .parseDouble(einlesenGeschwindigkeit);

            this.aktuellesFahrzeug = new Fahrzeug(xpos, ypos,
                    anzahlRaeder, gewicht, geschwindigkeit);

            this.ausgabeBereich
                    .setText("Folgendes Objekt wurde erzeugt: "
                            + this.aktuellesFahrzeug
                            + " mit der aktuellen Geschwindigkeit: "
                            + this.aktuellesFahrzeug
                                    .getAktuelleGeschwindigkeit());
        }
    }

    /**
     * Diese Methode löscht das aktuelle Fahrzeug, falls es eines gibt.
     */
    private void loeschen() {
        if (this.aktuellesFahrzeug == null) {
            this.ausgabeBereich
                    .setText("Es existiert kein Fahrzeug, "
                            + "das gelöscht werden kann.");
        }
        else {
            this.aktuellesFahrzeug = null;
            this.ausgabeBereich
                    .setText("Das Fahrzeug wurde gelöscht.");
        }
    }

    /**
     * Diese Methode beschleunigt das aktuelle Fahrzeug, falls es eines gibt.
     */
    private void beschleunigen() {
        if (this.aktuellesFahrzeug == null) {
            this.ausgabeBereich
                    .setText("Es existiert kein Fahrzeug, "
                            + "das beschleunigt werden kann.");

        }
        else {
            String einlesenBeschleunigung = JOptionPane
                    .showInputDialog("Um wieviel möchten Sie beschleunigen: ");
            double beschleunigung = Double
                    .parseDouble(einlesenBeschleunigung);

            this.aktuellesFahrzeug.beschleunigen(beschleunigung);

            this.ausgabeBereich.setText("Das Objekt: "
                    + this.aktuellesFahrzeug
                    + " hat nun die aktuelle Geschwindigkeit: "
                    + this.aktuellesFahrzeug
                            .getAktuelleGeschwindigkeit());
        }
    }
}
